/*
 * Copyright (C) 2019 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.akamai.downloader.sample;

import android.content.Intent;
import android.net.Uri;

import androidx.annotation.Nullable;

import com.akamai.amp.exoplayer2.util.Assertions;
import com.akamai.amp.exoplayer2.util.Util;

import java.util.UUID;

import static com.akamai.amp.media.exowrapper2.ExoTags.ACTION_VIEW;
import static com.akamai.amp.media.exowrapper2.ExoTags.AD_TAG_URI_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.DRM_KEY_REQUEST_PROPERTIES_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.DRM_LICENSE_URL_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.DRM_MULTI_SESSION_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.DRM_SCHEME_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.DRM_SCHEME_UUID_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.EXTENSION_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.IS_LIVE_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.SPHERICAL_STEREO_MODE_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.SUBTITLE_LANGUAGE_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.SUBTITLE_MIME_TYPE_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.SUBTITLE_URI_EXTRA;
import static com.akamai.amp.media.exowrapper2.ExoTags.URI_EXTRA;

/* package */ abstract class Sample {

  public static final class UriSample extends Sample {

    public final Uri uri;
    public final String extension;
    public final boolean isLive;
    @Nullable public final String sphericalStereoMode;
    @Nullable SubtitleInfo subtitleInfo;

    public UriSample(
        String name,
        Uri uri,
        String extension,
        boolean isLive,
        @Nullable String sphericalStereoMode,
        @Nullable SubtitleInfo subtitleInfo) {
      super(name);
      this.uri = uri;
      this.extension = extension;
      this.isLive = isLive;
      this.sphericalStereoMode = sphericalStereoMode;
      this.subtitleInfo = subtitleInfo;
    }

    @Override
    public void addToIntent(Intent intent) {
      intent.setAction(ACTION_VIEW).setData(uri);
      intent.putExtra(IS_LIVE_EXTRA, isLive);
      intent.putExtra(SPHERICAL_STEREO_MODE_EXTRA, sphericalStereoMode);
      addPlayerConfigToIntent(intent, /* extrasKeySuffix= */ "");
    }

    private void addPlayerConfigToIntent(Intent intent, String extrasKeySuffix) {
      intent
          .putExtra(EXTENSION_EXTRA + extrasKeySuffix, extension);
      if (subtitleInfo != null) {
        subtitleInfo.addToIntent(intent, extrasKeySuffix);
      }
    }
  }

  public static final class SubtitleInfo {

    @Nullable
    public static SubtitleInfo createFromIntent(Intent intent, String extrasKeySuffix) {
      if (!intent.hasExtra(SUBTITLE_URI_EXTRA + extrasKeySuffix)) {
        return null;
      }
      return new SubtitleInfo(
          Uri.parse(intent.getStringExtra(SUBTITLE_URI_EXTRA + extrasKeySuffix)),
          intent.getStringExtra(SUBTITLE_MIME_TYPE_EXTRA + extrasKeySuffix),
          intent.getStringExtra(SUBTITLE_LANGUAGE_EXTRA + extrasKeySuffix));
    }

    public final Uri uri;
    public final String mimeType;
    @Nullable public final String language;

    public SubtitleInfo(Uri uri, String mimeType, @Nullable String language) {
      this.uri = Assertions.checkNotNull(uri);
      this.mimeType = Assertions.checkNotNull(mimeType);
      this.language = language;
    }

    public void addToIntent(Intent intent, String extrasKeySuffix) {
      intent.putExtra(SUBTITLE_URI_EXTRA + extrasKeySuffix, uri.toString());
      intent.putExtra(SUBTITLE_MIME_TYPE_EXTRA + extrasKeySuffix, mimeType);
      intent.putExtra(SUBTITLE_LANGUAGE_EXTRA + extrasKeySuffix, language);
    }
  }

  @Nullable public final String name;

  public Sample(String name) {
    this.name = name;
  }

  public abstract void addToIntent(Intent intent);
}
